package org.qatools.util;

import org.apache.log4j.Logger;
import org.qatools.remote.RemoteConn;

public class StringUtil extends QAToolsUtil {
    private static Logger sLogger = Logger.getLogger(StringUtil.class);

    
    // ------------------------------------------------------------
    // ----- constants --------------------------------------------
    // ------------------------------------------------------------

    public static char[] NUMERIC_CHARS = { '0', '1', '2', '3', '4', '5', '6', '7', '8', '9' };
    public static char[] ALPHA_CHARS_LOWER = { 'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n', 'o',
                                         	   'p', 'q', 'r', 's', 't', 'u', 'v', 'w', 'x', 'y', 'z' };
    public static char[] ALPHA_CHARS_UPPER = { 'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M', 'N', 'O',
                                         	   'P', 'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X', 'Y', 'Z' };
    public static char[] ALPHANUMERIC_CHARS = { '0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'a', 'b', 'c', 'd',
                                               'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n', 'o', 'p', 'q', 'r',
                                               's', 't', 'u', 'v', 'w', 'x', 'y', 'z' };
    public static char[] SPECIAL_CHARS = { '~', '!', '@', '#', '$', '%', '^', '&', '*', '(', ')', '_', '-', '+', '=', '[', '}', '{', '}', '|', '<', '>', ',', '.', '/', '?', ';', ':' };

    
    
	// -------------------------------------------------------------
	// ----- isStrNullOrEmpty --------------------------------------
	// -------------------------------------------------------------

    public static boolean isStrNullOrEmpty(String str) {
    	if (str==null || str.length()==0) {
    		return true;
    	}
    	return false;
    }

    
	// -------------------------------------------------------------
	// ----- getPaddedValue ----------------------------------------
	// -------------------------------------------------------------


    /**
     * zero pads the value
     * 
     * ie "55" & 999 returns "055" 
     */
    public static String getPaddedValueMaxValue(int value, int maxValue) {
    	int mNumDigits = ("" + maxValue).length();
    	return getPaddedValueMaxDigits(value, mNumDigits);
    }

    /**
     * zero pads the value
     * 
     * ie "55" & 3 returns "055"
     */
    public static String getPaddedValueMaxDigits(int value, int numDigits) {
    	String mPaddedValue = "" + value;
    	int mActualSize = mPaddedValue.length();    	
    	if (numDigits <= mActualSize) {
    		return mPaddedValue;
    	}
    	
    	// pad string with zeros
    	int mDiff = numDigits - mActualSize;
    	for (int i=0; i<mDiff; i++) {
    		mPaddedValue = "0" + mPaddedValue;
    	}
    	
    	return mPaddedValue;
    }

    // --------------------------------------------------------------------
    // ----- String generation methods ------------------------------------
    // --------------------------------------------------------------------

    /**
     * Returns a random character from the array of chars
     * 
     */
    private static char getRandomChar(char[] chars) {
        return chars[IntUtil.getRandomInt(chars.length)];
    }

    /**
     * Generates a String of random characters.
     * 
     * @param length -
     *            length of the returned String
     * @param chars -
     *            array of chars to use. If null, defaults to alpha & numeric characters
     * @return
     */
    public static String getRandomString(int length, char[] chars) {
        StringBuffer mStringBuffer = new StringBuffer(length);

        try {
            for (int i = 0; i < length; i++) {
                char bChar;
                if ((chars == null) || (chars.length == 0)) {
                    bChar = getRandomChar(ALPHANUMERIC_CHARS);
                } else {
                    bChar = getRandomChar(chars);
                }

                mStringBuffer.append(bChar);
            }
        } catch (Exception ex) {
            sLogger.error(ex.getMessage());
        } finally {

        }

        return mStringBuffer.toString();
    }

    /**
     * Generates a String using alpha & numeric characters
     * 
     */
    public static String getRandomString(int length) {
        return getRandomString(length, ALPHANUMERIC_CHARS);
    }

    /**
     * 
     */
    public static String getRandomStringLowerCase(int length) {
        return getRandomString(length, ALPHA_CHARS_LOWER);
    }

    /**
     * 
     */
    public static String getRandomStringUpperCase(int length) {
        return getRandomString(length, ALPHA_CHARS_UPPER);
    }

    /**
     * 
     */
    public static String getRandomSpecialCharacterString(int length) {
        return getRandomString(length, SPECIAL_CHARS);
    }

    /**
     * Generates a String using numeric characters
     * 
     * Number can not start with "0"
     */
    public static String getRandomNumericString(int length) {
        String mRandNum=getRandomString(length, NUMERIC_CHARS);
        while (mRandNum.charAt(0) == '0') {
            mRandNum=getRandomString(length, NUMERIC_CHARS);
        }
        return mRandNum;
    }
    
    // --------------------------------------------------------------------
    // ----- getUniqueLong (returns String value) -------------------------
    // --------------------------------------------------------------------

    public static String getUniqueLong() {
    	return "" + LongUtil.getUniqueLong();
    }

    // --------------------------------------------------------------------
    // ----- getUniqueLong (returns String value) -------------------------
    // --------------------------------------------------------------------

    /**
     * Generates a String that looks like this:
     * 		50bc394d-da3d-41b5-8959-b8e49a2ae265
     * 
     * Requires a remote connection with the 'uuidgen' command in the path
     */
    public static String getUuidgen(RemoteConn rc) {
    	return SshUtil.runCommandGetResult(rc, "uuidgen");
    }
     
    
    
    
    
    
}
