package org.qatools.util;

import java.io.IOException;
import java.util.Map.Entry;
import java.util.Properties;

import org.apache.log4j.Logger;

public class SystemUtil extends QAToolsUtil {
    private static Logger logger = Logger.getLogger(SystemUtil.class);


    /**
     * Given the full path to a file in the classpath, 
     * this loads all properties in the file into System properties and logs each key / value pair.
     * 
     * @param file
     */
    public static void loadPropertiesFileIntoSystemProperties(String filename) throws IOException {
        Properties props = new Properties();	//System.getProperties();
        props.load(ClassLoader.getSystemResourceAsStream(filename));
        
        for (Entry<Object, Object> entry: props.entrySet()) {
            logger.debug("Loading '" + entry.getKey() + "' = '" + entry.getValue() + "'");
            System.setProperty((String)entry.getKey(), (String)entry.getValue());
        }
    }

 
    // ------------------------------------------------------
    // ----- optional
    // ------------------------------------------------------
    
    /**
     * This retrieves a system property, trims any leading or trailing spaces 
     * and converts to the proper data type.
     * 
     * @param key
     * @return
     */
    public static Object getOptionalObjectProperty(String key) {
    	Object obj = System.getProperty(key);
    	logger.debug("getOptionalObjectProperty(" + key + ") = " + obj);
    	return obj;
    }
    
    /**
     * 
     * @param key
     * @return
     */
    public static String getOptionalStringProperty(String key) {
    	Object obj = getOptionalObjectProperty(key);
    	if (obj == null) {
    		return null;
    	}
    	return ((String) obj).trim();
    }

    /**
     * 
     * @param key
     * @return
     */
    public static Long getOptionalLongProperty(String key) {
    	String str = getOptionalStringProperty(key);
    	if (str==null || str.trim().length()==0) {
    		return null;
    	}
    	return new Long(str);
    }

    /**
     * 
     * @param key
     * @return
     */
    public static Integer getOptionalIntegerProperty(String key) {
    	String str = getOptionalStringProperty(key);
    	if (str==null || str.trim().length()==0) {
    		return null;
    	}
    	return new Integer(str);
    }
    

    /**
     * 
     * @param key
     * @return
     */
    public static Boolean getOptionalBooleanProperty(String key) {
    	String str = getOptionalStringProperty(key);
    	if (str==null || str.trim().length()==0) {
    		return null;
    	}

    	Boolean bool = Boolean.FALSE;
    	if ("true".equals(str.toLowerCase().trim())) {
    		bool = Boolean.TRUE;
    	}
    	return bool;
    }

    // ------------------------------------------------------
    // ----- required
    // ------------------------------------------------------

    /**
     * 
     * @param key
     * @return
     * @throws Exception
     */
    public static Object getRequiredObjectProperty(String key) throws Exception {
    	Object obj = System.getProperty(key);
    	logger.debug("getRequiredObjectProperty(" + key + ") = " + obj);
    	if (obj == null) {
    		throw new Exception("Property '" + key + "' is not defined");
    	}
    	return obj;
    }

    /**
     * 
     * @param key
     * @return
     * @throws Exception
     */
    public static String getRequiredStringProperty(String key) throws Exception {
    	return ((String)getRequiredObjectProperty(key)).trim();
    }

    /**
     * 
     * @param key
     * @return
     * @throws Exception
     */
    public static Long getRequiredLongProperty(String key) throws Exception {
    	return new Long(getRequiredStringProperty(key));

    }

    /**
     * 
     * @param key
     * @return
     * @throws Exception
     */
    public static Integer getRequiredIntegerProperty(String key) throws Exception {
    	return new Integer(getRequiredStringProperty(key));
    }

    /**
     * 
     * @param key
     * @return
     * @throws Exception
     */
    public static Boolean getRequiredBooleanProperty(String key) throws Exception {
    	String value = getRequiredStringProperty(key);
    	Boolean bool = Boolean.FALSE;
    	if ("true".equals(value.toLowerCase().trim())) {
    		bool = Boolean.TRUE;
    	}
    	return bool;
    }

}
